@@ -151,6 +151,14 @@ class BitmexOrder:
             self.props[symbol] = {
                 'stoploss_id': '', 'stop_id': '', 'entry': .0, 'SL': .0, 'TP': .0, 'open': False, 'wait_stop': False, 'qty': 0}
 
+        self.show_slippage = False # only for testing
+        if self.show_slippage:
+            self.entry_acc_balance = {}
+            self.between_profits = {}
+            for symbol in symbols:
+                        self.between_profits[symbol] = .0
+                        self.entry_acc_balance[symbol] = .0
+
     def check_open_bot_order(self, symbol, entry, sl):
         settings = pickle.load(open(self._settings_path, 'rb'))
         self.props[symbol]['stoploss_id'] = settings.symbols[symbol]['last_order_stoploss_id']
@@ -187,7 +195,7 @@ class BitmexOrder:
             self.props[symbol]['stop_id'] = id
             settings.symbols[symbol]['last_order_stop_id'] = id
             pickle.dump(settings, file=open(self._settings_path, 'wb'))
-        elif stop_type == 'stoploss':
+        elif stop_type == 'stoploss': # also save the stoploss
             self.props[symbol]['stoploss_id'] = id
             settings.symbols[symbol]['last_order_stoploss_id'] = id
             settings.symbols[symbol]['last_order_SL'] = self.props[symbol]['SL']
@@ -195,44 +203,98 @@ class BitmexOrder:
         else:
             print("No Stop Type given.")
 
-    def manage_entries(self, symbols, me_count, cp=''):
-        for symbol in symbols:
-            if not cp:
-                cp = self.client.get_current_price(symbol)
-            if self.props[symbol]['wait_stop']:
-                time.sleep(0.5)
-                if self.props[symbol]['qty'] > 0 and cp >= self.props[symbol]['entry'] and self.client.open_contracts(symbol) != 0:
-                    self.props[symbol]['wait_stop'] = False
-                    self.props[symbol]['open'] = True
-                    print("Long Position has been opened in {}, amending Stoploss to".format(symbol),
-                          self.props[symbol]['SL'], flush=True)
-                    self.stoploss_order(symbol)
-                elif self.props[symbol]['qty'] < 0 and cp <= self.props[symbol]['entry'] and self.client.open_contracts(symbol) != 0:
-                    self.props[symbol]['wait_stop'] = False
-                    self.props[symbol]['open'] = True
-                    print("Short Position has been opened in {}, amending Stoploss to".format(symbol),
-                          self.props[symbol]['SL'], flush=True)
-                    self.stoploss_order(symbol)
-            elif self.props[symbol]['open']:
-                if self.props[symbol]['qty'] > 0 and cp <= self.props[symbol]['SL'] and self.client.open_contracts(symbol) == 0:
-                    self.props[symbol]['open'] = False
-                    print("Long Position {} closed. {} Contracts from {} to {}. PnL in Points: {}".format(symbol,
-                                                                                                          self.props[symbol]['qty'], self.props[symbol]['entry'], self.props[symbol]['SL'], self.price_decimals(symbol, self.props[symbol]['SL']-self.props[symbol]['entry'])), flush=True)
-                    self.props[symbol]['qty'] = 0
-                elif self.props[symbol]['qty'] < 0 and cp >= self.props[symbol]['SL'] and self.client.open_contracts(symbol) == 0:
-                    self.props[symbol]['open'] = False
-                    print("Short Position {} closed. {} Contracts from {} to {}. PnL in Points: {}".format(symbol,
-                                                                                                           -self.props[symbol]['qty'], self.props[symbol]['entry'], self.props[symbol]['SL'], self.price_decimals(symbol, self.props[symbol]['entry']-self.props[symbol]['SL'])), flush=True)
-                    self.props[symbol]['qty'] = 0
-         # every 3rd time also check the minute high/low (10 second window can miss executions sometimes)
-        if me_count % 3 == 0:
-            time.sleep(5)  # don't execute at same time like 'cp'
+    def set_position_open(self, symbol, dir):
+        self.props[symbol]['wait_stop'] = False
+        self.props[symbol]['open'] = True
+        print("{} Position has been opened in {}, amending Stoploss to".format(dir, symbol),
+              self.props[symbol]['SL'], flush=True)
+        self.stoploss_order(symbol)
+
+    def set_position_closed(self, symbol, dir, currentPrice=False):
+        self.props[symbol]['open'] = False
+        pnl_points = self.price_decimals(
+            symbol, self.props[symbol]['SL']-self.props[symbol]['entry'])
+        qty = self.props[symbol]['qty']
+        if dir == "Short":
+            pnl_points = -pnl_points
+            qty = -qty
+        if currentPrice:
+            exitPrice = self.client.get_current_price(symbol)
+        else:
+            exitPrice = self.props[symbol]['SL']
+        print("{} Position {} closed. {} Contracts from {} to {}. PnL in Points: {}".format(symbol,
+                                                                                            dir, qty,
+                                                                                            self.props[symbol]['entry'],
+                                                                                            exitPrice,
+                                                                                            pnl_points), flush=True)
+        if self.show_slippage and self.props[symbol]['initialSL']:
+            acc_pos_size = float(self.settings.symbols[symbol]['position_size'])/100
+
+            current_acc_balance = float(self.client.acc_balance)
+
+            between_profs = .0
+            for sym in self._symbols:
+                if symbol != sym:
+                    between_profs += self.between_profits[sym]
+                    self.between_profits[sym] = .0
+
+            # real acc balance difference counting the between profits
+            acc_balance_diff = self.entry_acc_balance[symbol] - current_acc_balance + between_profs
+            
+            self.between_profits[symbol] += acc_balance_diff # in acc balance difference
+
+            initial_risk_points = abs(self.props[symbol]['initialSL'] - self.props[symbol]['entry'])
+            pnl_rr = pnl_points / initial_risk_points
+            
+            pnl_rel_expected = self.entry_acc_balance[symbol] * (1 + (pnl_rr * acc_pos_size))
+            pnl_rel_real = self.entry_acc_balance[symbol] * (1 + (acc_balance_diff / self.entry_acc_balance[symbol]))
+
+            # slippage is real acc balance change relative - exepceted pnl percent
+            print("Slippage: {}%".format(round((pnl_rel_real - pnl_rel_expected) * 100,2)),flush=True)
+            self.entry_acc_balance[symbol] = current_acc_balance
+        self.props[symbol]['qty'] = 0
+
+    def has_position_opened(self, symbol, cp, candlePrice):
+        if self.client.open_contracts(symbol) != 0:
             if self.props[symbol]['qty'] > 0:
-                cp = self.client.get_current_price_candle(symbol)['high']
-                self.manage_entries(self._symbols, me_count+1, cp=cp)
+                _cp = candlePrice['high'] if candlePrice else cp
+                if _cp >= self.props[symbol]['entry']:
+                    self.set_position_open(symbol, 'Long')
             elif self.props[symbol]['qty'] < 0:
-                cp = self.client.get_current_price_candle(symbol)['low']
-                self.manage_entries(self._symbols, me_count+1, cp=cp)
+                _cp = candlePrice['low'] if candlePrice else cp
+                if _cp <= self.props[symbol]['entry']:
+                    self.set_position_open(symbol, 'Short')
+
+    def has_position_closed(self, symbol, cp, candlePrice):
+        if self.client.open_contracts(symbol) == 0:
+            if self.props[symbol]['qty'] > 0:
+                _cp = candlePrice['low'] if candlePrice else cp
+                if _cp <= self.props[symbol]['SL']:
+                    self.set_position_closed(symbol, 'Long')
+            elif self.props[symbol]['qty'] < 0:
+                _cp = candlePrice['low'] if candlePrice else cp
+                if _cp >= self.props[symbol]['SL']:
+                    self.set_position_closed(symbol, 'Short')
+
+    def manage_entries(self, symbols, me_count=0, candlePrice=''):
+        for symbol in symbols:
+            if not candlePrice:
+                cp = self.client.get_current_price(symbol)
+            time.sleep(0.5)
+            if self.props[symbol]['wait_stop']:
+                self.has_position_opened(symbol, cp, candlePrice)
+            elif self.props[symbol]['open']:
+                self.has_position_closed(symbol, cp, candlePrice)
+
+        # every 30 seconds additional candle check to not miss entry/exit
+        if me_count and me_count % 3 == 0:
+            if self.props[symbol]['qty'] > 0:
+                candlePrice = self.client.get_current_price_candle(symbol)
+                # + 1 to prevent endless loop
+                self.manage_entries(self._symbols, candlePrice=cp)
+            elif self.props[symbol]['qty'] < 0:
+                candlePrice = self.client.get_current_price_candle(symbol)
+                self.manage_entries(self._symbols, candlePrice=cp)
 
     def generate_id(self, symbol, _type='stop'):
         return "{}_{}_{}_{}".format(self.magic, symbol, str(uuid.uuid4().fields[-1])[:5], _type)
@@ -247,12 +309,13 @@ class BitmexOrder:
             sys.exit()
 
     def calc_pos_size(self, symbol, sl_distance):
-        acc_balance = self.client.acc_balance
+        acc_balance = float(self.client.acc_balance)
+        self.entry_acc_balance = acc_balance
         ps = float(self.settings.symbols[symbol]['position_size'])
         if symbol == 'XBTUSD':
             btc_usd = self.client.last_current_price(symbol)
             lot_step = 100
-            riskvalue = (float(acc_balance) / 100000000) * \
+            riskvalue = (acc_balance / 100000000) * \
                 float(btc_usd) * (ps/100)  # riskvalue in usd
             pos_size = int(
                 round((math.floor(riskvalue / (sl_distance / btc_usd)) / lot_step)) * lot_step)
@@ -302,8 +365,7 @@ class BitmexOrder:
     def bracket_stop_order(self, symbol, orderQty, entry, sl, tp=0):
         try:
             if self.props[symbol]['qty'] != 0:
-                # order is open (shouldn't be)
-                # then close old order first
+                # order is open (shouldn't be) > close old order first
                 self.close(symbol)
                 time.sleep(1)
             self.props[symbol]['stop_id'] = self.generate_id(symbol)
@@ -315,6 +377,7 @@ class BitmexOrder:
             self.props[symbol]['qty'] = orderQty
             self.props[symbol]['wait_stop'] = True
             self.props[symbol]['SL'] = sl
+            self.props[symbol]['initialSL'] = sl
             self.props[symbol]['TP'] = tp
             self.props[symbol]['entry'] = entry
             return order_res
@@ -326,6 +389,7 @@ class BitmexOrder:
         try:
             entry = self.market_order(symbol, orderQty)
             self.props[symbol]['SL'] = sl
+            self.props[symbol]['initialSL'] = sl
             self.props[symbol]['TP'] = tp
             time.sleep(5)
             stoploss = self.stoploss_order(symbol)
@@ -374,18 +438,14 @@ class BitmexOrder:
                 close = self.client.client.Order.Order_new(
                     symbol=symbol, ordType='Market', execInst='Close', orderQty=qty
                 ).result()
-                time.sleep(2)  # wait before making new API call
-                self.stoploss_cancel(symbol)
+                time.sleep(1)  # wait before making new API call
                 if self.props[symbol]['qty'] > 0:
-                    self.props[symbol]['open'] = False
-                    print("Long Position {} closed. {} Contracts from {} to {}. PnL in Points: {}".format(symbol,
-                                                                                                          self.props[symbol]['qty'], self.props[symbol]['entry'], self.props[symbol]['SL'], self.price_decimals(symbol, self.props[symbol]['SL']-self.props[symbol]['entry'])), flush=True)
-                    self.props[symbol]['qty'] = 0
+                    self.set_position_closed(
+                        symbol, 'Long', currentPrice=True)
                 elif self.props[symbol]['qty'] < 0:
-                    self.props[symbol]['open'] = False
-                    print("Short Position {} closed. {} Contracts from {} to {}. PnL in Points: {}".format(symbol,
-                                                                                                           -self.props[symbol]['qty'], self.props[symbol]['entry'], self.props[symbol]['SL'], self.price_decimals(symbol, self.props[symbol]['entry']-self.props[symbol]['SL'])), flush=True)
-                    self.props[symbol]['qty'] = 0
+                    self.set_position_closed(
+                        symbol, 'Short', currentPrice=True)
+                self.stoploss_cancel(symbol)
                 return close
             except HTTPBadRequest as e:
                 print("Error closing order in {}.".format(symbol), e, flush=True)
