@@ -222,7 +222,7 @@ class Lotus:
         while True:
             time.sleep(10)
             self.me_count += 1
-            self.order.manage_entries(self.symbols, self.me_count)
+            self.order.manage_entries(self.symbols, me_count=self.me_count)
             now = datetime.datetime.utcnow()
             if now.minute % 30 == 0:
                 exec_minute = now.minute
@@ -244,6 +244,7 @@ class Lotus:
                     time.sleep(self.rand_sec())
                 while True:
                     time.sleep(10)
-                    self.order.manage_entries(self.symbols)
+                    self.order.manage_entries(
+                        self.symbols, me_count=self.me_count)
                     if datetime.datetime.utcnow().minute != exec_minute:
                         break
